#include "KShape.h"
#include "KCircle.h"
#include "KRect.h"

#include <stdexcept>
#include <typeinfo>

namespace KapitanGame {
	std::atomic<int> KShape::IdCounter{ 0 };

	KShape::KShape(const KVector2D& position, const KVector2D& velocity, const KTexture& texture) : Id(++IdCounter),
		Position(position),
		Velocity(velocity),
		Texture(texture) {
	}

	KShape::~KShape() = default;

	void KShape::CollisionDetectionStep(const std::vector<KShape>& shapes, const bool& shouldSeparate,
		const bool& shouldReflect) {
		for (auto& other : shapes) {
			if (other.Id == Id) continue;
			if (IsCollision(other)) {
				auto separationVector = GetSeparationVector(other);
				if (shouldSeparate)
					Position += separationVector;
				if (shouldReflect)
				{
					separationVector.Normalize();
					Velocity = Velocity.Reflect(separationVector);
				}
			}
		}
	}

	bool KShape::IsCollision(const KShape& other) const {
		try {
			auto& circle = dynamic_cast<const KCircle&>(other);
			return IsCollision(circle);
		}
		catch (const std::bad_cast&) {
		}
		try {
			auto& rect = dynamic_cast<const KRect&>(other);
			return IsCollision(rect);
		}
		catch (const std::bad_cast&) {
		}
		throw std::runtime_error("unsupported shape");
	}

	KVector2D KShape::GetSeparationVector(const KShape& other) const {
		try {
			auto& circle = dynamic_cast<const KCircle&>(other);
			return GetSeparationVector(circle);
		}
		catch (const std::bad_cast&) {
		}
		try {
			auto& circle = dynamic_cast<const KRect&>(other);
			return GetSeparationVector(circle);
		}
		catch (const std::bad_cast&) {
		}
		throw std::runtime_error("unsupported shape");
	}

	void KShape::MovementStep(const float& timeStep) {
		Position += Velocity * timeStep;
	}

	void KShape::Render(SDL_Renderer* renderer) const {
		Texture.Render(renderer, static_cast<int>(Position.X) - Texture.GetWidth() / 2, static_cast<int>(Position.Y) - Texture.GetHeight() / 2);
	}

	KVector2D KShape::GetPosition() const {
		return Position;
	}
}
